@@ -50,17 +50,27 @@ for (method in methods[6:8]){
   expect_equal(stringsim(x,y,method="jw", useBytes=FALSE), 1-1/3)
   expect_equal(stringsim(x,y,method="jw", useBytes=TRUE ), (1/2 + 1/3 +1)/3)
 
+# stringsimmatrix
+  x <- names(islands)[1:10] 
+  y <- rev(x) # o-umlaut
+  expect_true(inherits(stringsimmatrix(x,y,method="osa", useBytes=FALSE), "matrix"))
+  expect_equal(dim(stringsimmatrix(x,y,method="osa", useBytes=FALSE)), c(10, 10))
+  expect_equal(stringsimmatrix(x,y,method="osa", useBytes=FALSE)[2, 2], 0.2)
+  expect_true(inherits(stringsimmatrix(x,method="osa", useBytes=FALSE), "matrix"))
+  expect_equal(dim(stringsimmatrix(x,method="osa", useBytes=FALSE)), c(10, 10))
+  expect_equal(stringsimmatrix(x,method="osa", useBytes=FALSE)[2, 9], 0.2)
+  expect_warning(stringdistmatrix(list('a')))
+  expect_warning(stringdistmatrix(list('a'),list('b')))
+  
 ## seq_sim
 
-## elementary seq_sim test
-#
-# There seams to be an infrequently occurring edge case 
-# when 2 threads are used. We turn it of for now so we can
-# keep stringdist on CRAN. 
-options(sd_num_thread=1)
+# We used to have list(1:3, 2:4) and list(1:3). This occasionally
+# gave failing tests, and only in the context of expect_equal (both
+# for tinytest and testthat. Therefore this may point to a hard-to-reproduce
+# bug in R's JIT compiler.
   expect_equal(
-    seq_sim(list(1:3,2:4),list(1:3))
-     , stringsim(c("abc","bcd"),"abc") 
+       seq_sim(list(c(1,2,3),c(2,3,4)), list(c(1,2,3)), method="cosine")
+     , stringsim(c("abc","bcd"),"abc", method="cosine") 
   )
 
 
